@@ -117,22 +117,20 @@ static void usage(const char *argv0)
 		switch (cmd->idby) {
 		case CIB_NONE:
 			fprintf(stderr, "\t");
-			/* fall through */
+			break;
 		case CIB_PHY:
-			if (cmd->idby == CIB_PHY)
-				fprintf(stderr, "\tphy <phyname> ");
-			/* fall through */
+			fprintf(stderr, "\tphy <phyname> ");
+			break;
 		case CIB_NETDEV:
-			if (cmd->idby == CIB_NETDEV)
-				fprintf(stderr, "\tdev <devname> ");
-			if (cmd->section)
-				fprintf(stderr, "%s ", cmd->section);
-			fprintf(stderr, "%s", cmd->name);
-			if (cmd->args)
-				fprintf(stderr, " %s", cmd->args);
-			fprintf(stderr, "\n");
+			fprintf(stderr, "\tdev <devname> ");
 			break;
 		}
+		if (cmd->section)
+			fprintf(stderr, "%s ", cmd->section);
+		fprintf(stderr, "%s", cmd->name);
+		if (cmd->args)
+			fprintf(stderr, " %s", cmd->args);
+		fprintf(stderr, "\n");
 	}
 }
 
@@ -149,6 +147,8 @@ static int phy_lookup(char *name)
 	snprintf(buf, sizeof(buf), "/sys/class/ieee80211/%s/index", name);
 
 	fd = open(buf, O_RDONLY);
+	if (fd < 0)
+		return -1;
 	pos = read(fd, buf, sizeof(buf) - 1);
 	if (pos < 0)
 		return -1;
@@ -178,28 +178,44 @@ static int ack_handler(struct nl_msg *msg, void *arg)
 	return NL_STOP;
 }
 
-static int handle_cmd(struct nl80211_state *state,
-		      enum command_identify_by idby,
-		      int argc, char **argv)
+int handle_cmd(struct nl80211_state *state, enum id_input idby,
+	       int argc, char **argv)
 {
-	struct cmd *cmd;
-	struct nl_cb *cb = NULL;
+	struct cmd *cmd, *match = NULL;
+	struct nl_cb *cb;
 	struct nl_msg *msg;
 	int devidx = 0;
-	int err;
+	int err, o_argc;
 	const char *command, *section;
+	char *tmp, **o_argv;
+	enum command_identify_by command_idby = CIB_NONE;
 
-	if (argc <= 1 && idby != CIB_NONE)
+	if (argc <= 1 && idby != II_NONE)
 		return 1;
 
+	o_argc = argc;
+	o_argv = argv;
+
 	switch (idby) {
-	case CIB_PHY:
+	case II_PHY_IDX:
+		command_idby = CIB_PHY;
+		devidx = strtoul(*argv + 4, &tmp, 0);
+		if (*tmp != '\0')
+			return 1;
+		argc--;
+		argv++;
+		break;
+	case II_PHY_NAME:
+		command_idby = CIB_PHY;
 		devidx = phy_lookup(*argv);
 		argc--;
 		argv++;
 		break;
-	case CIB_NETDEV:
+	case II_NETDEV:
+		command_idby = CIB_NETDEV;
 		devidx = if_nametoindex(*argv);
+		if (devidx == 0)
+			devidx = -1;
 		argc--;
 		argv++;
 		break;
@@ -207,6 +223,9 @@ static int handle_cmd(struct nl80211_state *state,
 		break;
 	}
 
+	if (devidx < 0)
+		return -errno;
+
 	section = command = *argv;
 	argc--;
 	argv++;
@@ -215,15 +234,18 @@ static int handle_cmd(struct nl80211_state *state,
 	     cmd = (struct cmd *)((char *)cmd + cmd_size)) {
 		if (!cmd->handler)
 			continue;
-		if (cmd->idby != idby)
+		if (cmd->idby != command_idby)
 			continue;
 		if (cmd->section) {
 			if (strcmp(cmd->section, section))
 				continue;
 			/* this is a bit icky ... */
 			if (command == section) {
-				if (argc <= 0)
+				if (argc <= 0) {
+					if (match)
+						break;
 					return 1;
+				}
 				command = *argv;
 				argc--;
 				argv++;
@@ -234,12 +256,21 @@ static int handle_cmd(struct nl80211_state *state,
 			continue;
 		if (argc && !cmd->args)
 			continue;
-		break;
+
+		match = cmd;
 	}
 
-	if (cmd >= &__stop___cmd)
+	cmd = match;
+
+	if (!cmd)
 		return 1;
 
+	if (!cmd->cmd) {
+		argc = o_argc;
+		argv = o_argv;
+		return cmd->handler(state, NULL, NULL, argc, argv);
+	}
+
 	msg = nlmsg_alloc();
 	if (!msg) {
 		fprintf(stderr, "failed to allocate netlink message\n");
@@ -256,7 +287,7 @@ static int handle_cmd(struct nl80211_state *state,
 	genlmsg_put(msg, 0, 0, genl_family_get_id(state->nl80211), 0,
 		    cmd->nl_msg_flags, cmd->cmd, 0);
 
-	switch (idby) {
+	switch (command_idby) {
 	case CIB_PHY:
 		NLA_PUT_U32(msg, NL80211_ATTR_WIPHY, devidx);
 		break;
@@ -267,7 +298,7 @@ static int handle_cmd(struct nl80211_state *state,
 		break;
 	}
 
-	err = cmd->handler(cb, msg, argc, argv);
+	err = cmd->handler(state, cb, msg, argc, argv);
 	if (err)
 		goto out;
 
@@ -368,11 +399,33 @@ static int print_event(struct nl_msg *msg, void *arg)
 	return NL_SKIP;
 }
 
-static int listen_events(struct nl80211_state *state,
-			 int argc, char **argv)
+struct wait_event {
+	int n_cmds;
+	const __u32 *cmds;
+	__u32 cmd;
+};
+
+static int wait_event(struct nl_msg *msg, void *arg)
+{
+	struct wait_event *wait = arg;
+	struct genlmsghdr *gnlh = nlmsg_data(nlmsg_hdr(msg));
+	int i;
+
+	for (i = 0; i < wait->n_cmds; i++) {
+		if (gnlh->cmd == wait->cmds[i]) {
+			wait->cmd = gnlh->cmd;
+		}
+	}
+
+	return NL_SKIP;
+}
+
+__u32 listen_events(struct nl80211_state *state,
+		    const int n_waits, const __u32 *waits)
 {
 	int mcid, ret;
 	struct nl_cb *cb = nl_cb_alloc(debug ? NL_CB_DEBUG : NL_CB_DEFAULT);
+	struct wait_event wait_ev;
 
 	if (!cb) {
 		fprintf(stderr, "failed to allocate netlink callbacks\n");
@@ -406,14 +459,23 @@ static int listen_events(struct nl80211_state *state,
 
 	/* no sequence checking for multicast messages */
 	nl_cb_set(cb, NL_CB_SEQ_CHECK, NL_CB_CUSTOM, no_seq_check, NULL);
-	nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, print_event, NULL);
 
-	while (1)
+	if (n_waits && waits) {
+		wait_ev.cmds = waits;
+		wait_ev.n_cmds = n_waits;
+		nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, wait_event, &wait_ev);
+	} else {
+		nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, print_event, NULL);
+	}
+
+	wait_ev.cmd = 0;
+
+	while (!wait_ev.cmd)
 		nl_recvmsgs(state->nl_sock, cb);
 
 	nl_cb_put(cb);
 
-	return 0;
+	return wait_ev.cmd;
 }
 
 int main(int argc, char **argv)
@@ -450,17 +512,25 @@ int main(int argc, char **argv)
 		return 1;
 
 	if (strcmp(*argv, "event") == 0) {
-		err = listen_events(&nlstate, argc, argv);
+		if (argc != 1)
+			err = 1;
+		else
+			err = listen_events(&nlstate, 0, NULL);
 	} else if (strcmp(*argv, "dev") == 0) {
 		argc--;
 		argv++;
-		err = handle_cmd(&nlstate, CIB_NETDEV, argc, argv);
-	} else if (strcmp(*argv, "phy") == 0) {
-		argc--;
-		argv++;
-		err = handle_cmd(&nlstate, CIB_PHY, argc, argv);
+		err = handle_cmd(&nlstate, II_NETDEV, argc, argv);
+	} else if (strncmp(*argv, "phy", 3) == 0) {
+		if (strlen(*argv) == 3) {
+			argc--;
+			argv++;
+			err = handle_cmd(&nlstate, II_PHY_NAME, argc, argv);
+		} else if (*(*argv + 3) == '#')
+			err = handle_cmd(&nlstate, II_PHY_IDX, argc, argv);
+		else
+			err = 1;
 	} else
-		err = handle_cmd(&nlstate, CIB_NONE, argc, argv);
+		err = handle_cmd(&nlstate, II_NONE, argc, argv);
 
 	if (err == 1)
 		usage(argv0);
